//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __UPV__
#define __UPV__

#include <IO.h>
#include <Coinc2.h>
#include <VetoPerf.h>
#include "Uconfig.h"

using namespace std;

/**
 * Store UPV source parameters.
 * \author    Florent Robinet
 */
class UPVsource{

 public:
  UPVsource(void);         ///< Constructor
  virtual ~UPVsource(void);///< Destructor

  string source_files;     ///< source trigger file pattern (full sample)
  string source_name;      ///< channel name
  string outdir;           ///< output directory
  int target_nc;           ///< number of active target clusters
  int source_nc;           ///< number of active source clusters
  double source_livetime;  ///< source livetime
  double coinc_livetime;   ///< coinc livetime
  int source_coinc_nc;     ///< number of source clusters involved in a coinc
  int target_coinc_nc;     ///< number of target clusters involved in a coinc
  double veto_dt;          ///< veto deadtime
  int veto_nseg;           ///< number of veto segments
  int source_veto_nc;      ///< number of source clusters above veto thr
  int source_veto_u;       ///< veto number of used source clusters
  int target_veto_u;       ///< veto number of used target clusters
  bool status;             ///< status
};

/**
 * Study and characterize coupling between 2 trigger data sets.
 * The coupling is characterized using 'monitors' which are histograms measuring the trigger parameters as a function of the trigger frequency. This is the reason why this class inherits from the Coinc2 class.
 *
 * After the coupling characterization, this class offers many functions to design a veto targeting the target triggers and using the source triggers.
 * \author    Florent Robinet
 */
class UPV{

 public:

  /**
   * @name Constructors and destructors
   @{
  */
  /**
   * Constructor of the UPV class.
   * The option file is parsed. Target triggers are loaded and clustered. The coincidence algorithm is initialized. Coupling monitors are initialized.
   * @param aOptFile path to option file
   * @param aInSeg pointer to input segments
   */
  UPV(const string aOptFile, Segments *aInSeg);

  /**
   * Destructor of the UPV class.
   */
  virtual ~UPV(void);
  /**
     @}
  */

  /**
   * Loads the source trigger set with a given index.
   * Source triggers are first clustered.
   * @param aSourceIndex source index
  */
  bool LoadSourceTriggers(const int aSourceIndex);

  /**
   * Returns the number of available source channels.
   */
  inline int GetNsource(void){ return (int)source_trigfiles.size(); };
    
  /**
   * Wrapper for Coinc2::MakeCoinc().
   * Returns the number of coincs. Returns -1 if this function fails.
   */
  inline int MakeCoinc(void){
    if(source_index>=0){
      source_results[source_index]->target_coinc_nc = C2->MakeCoinc0();
      return source_results[source_index]->target_coinc_nc;
    }
    return -1;
  }
    
  /**
   * Fills the source/target monitors.
   * Once the source triggers have been defined with LoadSourceTriggers(), this function fills the trigger monitors using the valid clusters (tag>=0).
   * @param aTriggerIndex trigger index, 0 for source, 1 for target
   */
  bool MakeTriggerMonitors(const int aTriggerIndex);

  /**
   * Fills the coinc monitors.
   * Once the source triggers have been defined with LoadSourceTriggers() and set in coincidence with MakeCoinc(), this function fills the coinc monitors using all coinc events.
   * Please note that the overlap time include the coincidence time window and some optional time offset.
   */
  bool MakeCoincMonitors(void);

  /**
   * Fills the proc monitors.
   */
  void MakeProcMonitors(void);
  
  /**
   * Prints monitors as plots in PNG files.
   * Files are saved in the source directory.
   */
  void PrintMonitors();
  
  /**
   * Prints veto segments.
   * File is saved in the source directory.
   */
  void PrintVeto(void);
  
  /**
   * Tune veto threshold.
   * The veto threshold is a SNR threshold which is frequency dependent. It applies to the source triggers. The veto threshold is tuned to have a minimum source use-time percentage and a minimum number of coincident clusters, as requested from the option file. The veto dead-time is returned.
   */
  double MakeVetoThreshold(void);

  /**
   * Remove vetoed target triggers from analysis segments.
   * For each target vetoed triggers, the segment tstart-tend is removed from the analysis segments. This is useful when generating multiple vetoes in a row (hierarchical loop).
   */
  bool RemoveVetoedTriggers(void);

  /**
   * Returns the list of veto segments.
   * Returns a pointer to NULL if this function fails.
   * Delete the returned array.
   */
  Segments* GetVetoSegments(void);
  
  /**
   * Generates a html report.
   * A list of source indexes must be provided to tell how sort channels.
   * @param aIndexes sorted list of source indexes
   */
  bool MakeHtml(vector <int> aIndexes);

  /**
   * Returns the list of source indexes sorted by decreasing values of veto time efficiency.
   */
  vector <int> Sort(void);
  
  /**
   * Returns the source index for the channel with the best veto time efficiency.
   */
  int GetWinIndex(void);

  /**
   * Returns the target stream name.
   */
  inline string GetTargetName(void){ return target->GetName(); };

  /**
   * Returns the source stream name.
   */
  inline string GetSourceName(void){ if(source==NULL) return ""; return source->GetName(); };
  
  /**
   * Returns the source channel status.
   * @param aSourceIndex source index
   */ 
  inline bool GetSourceStatus(const int aSourceIndex){ return source_results[aSourceIndex]->status; };

  /**
   * Sets the source channel status.
   * @param aSourceIndex source index
   * @param aNewStatus new status value
   */ 
  inline void SetSourceStatus(const int aSourceIndex, const bool aNewStatus){ source_results[aSourceIndex]->status=aNewStatus; return; };

  /**
   * Returns the veto efficiency.
   * @param aSourceIndex source index
   */ 
  inline double GetVetoEfficiency(const int aSourceIndex){ return (double)(source_results[aSourceIndex]->target_veto_u)/(double)(source_results[aSourceIndex]->target_nc); };

  /**
   * Returns the number of active source channels.
   */ 
  inline int GetSourceNactive(void){
    int n=0;
    for(int i=0; i<GetNsource(); i++) n+=(int)(source_results[i]->status);
    return n;
  };

  /**
   * Adds a subtitle in the html output.
   * @param SetSubTitle subtitle string
   */ 
  inline void SetSubTitle(const string aSubTitle){ subtitle=aSubTitle; };

 private:

  bool status_OK;                      ///< class status
  time_t timer_start;                  ///< timer start

  // GENERAL
  string optionfile;                   ///< parameter file
  bool GetParameters(const string aOptFile, Segments *aInSeg);///< parse option file
  Segments *insegments;                ///< analysis input segments (DO NOT DELETE)
  Segments *cursegments;               ///< analysis current segments
  string outdir;                       ///< main output directory
  string subtitle;                     ///< subtitle
  int verbosity;                       ///< verbosity level

  void ResetMonitors(void);            ///< reset all monitors

  // COINC
  Coinc2 *C2;                          ///< coinc object
  TGraph *snrsnr;                      ///< coinc snr-snr plot
  double C2_dt;                        ///< coinc time window

  // SOURCE
  ReadTriggers *source;                ///< source triggers
  int source_index;                    ///< current source index
  double source_clusterdt;             ///< time clustering dt for source triggers [s]
  vector <string> source_trigfiles;    ///< source trigger file pattern (full sample)
  UPVsource **source_results;          ///< source results
  TH2D *source_c,  *source_c_cum;      ///< source clusters
  TH2D *source_ct, *source_ct_cum;     ///< source cluster time
  TH2D *source_co, *source_co_cum;     ///< coincs - source ref
  TH2D *source_up, *source_up_cum;     ///< source use-percentage

  // TARGET
  string target_trigfiles;             ///< target trigger file pattern (full sample)
  double target_clusterdt;             ///< time clustering dt for target triggers [s]
  double target_snrmin;                ///< SNR threshold for target clusters
  double target_freqmin;               ///< minimum frequency
  double target_freqmax;               ///< maximum frequency
  TriggerPlot *target;                 ///< target triggers
  vector <double> snrthr;              ///< snr threshold for VetoPerf
  TH2D *target_c,  *target_c_cum;      ///< target clusters
  TH2D *target_co, *target_co_cum;     ///< coincs - source ref

  // VETO
  int     veto_umin;                   ///< veto use min
  double veto_upmin;                   ///< veto up min
  TH1D *veto_thr;                      ///< veto threshold
  vector <int> veto_perf;              ///< veto perf flag
  VetoPerf *vp;                        ///< veto perf object

  ClassDef(UPV,0)  
};


#endif


